@@ -1,11 +1,4 @@
-# $Id: NodeList.pm 776 2009-02-06 11:12:56Z pajas $
-#
-# This is free software, you may use it and distribute it under the same terms as
-# Perl itself.
-#
-# Copyright 2001-2003 AxKit.com Ltd., 2002-2006 Christian Glahn, 2006-2009 Petr Pajas
-#
-#
+# $Id: NodeList.pm,v 1.2 2002/03/09 14:19:52 phish Exp $
 
 package XML::LibXML::NodeList;
 use strict;
@@ -13,9 +6,6 @@ use XML::LibXML::Boolean;
 use XML::LibXML::Literal;
 use XML::LibXML::Number;
 
-use vars qw ($VERSION);
-$VERSION = "1.69_2"; # VERSION TEMPLATE: DO NOT CHANGE
-
 use overload 
 		'""' => \&to_literal,
                 'bool' => \&to_boolean,
@@ -26,11 +16,6 @@ sub new {
 	bless [@_], $class;
 }
 
-sub new_from_ref {
-	my ($class,$array_ref,$reuse) = @_;
-	return bless $reuse ? $array_ref : [@$array_ref], $class;
-}
-
 sub pop {
 	my $self = CORE::shift;
 	CORE::pop @$self;
@@ -68,9 +53,7 @@ sub size {
 	scalar @$self;
 }
 
-sub get_node {
-    # uses array index starting at 1, not 0
-    # this is mainly because of XPath.
+sub get_node { # uses array index starting at 1, not 0
 	my $self = CORE::shift;
 	my ($pos) = @_;
 	$self->[$pos - 1];
@@ -109,11 +92,6 @@ sub to_number {
 			);
 }
 
-sub iterator {
-    warn "this function is obsolete!\nIt was disabled in version 1.54\n";
-    return undef;
-}
-
 1;
 __END__
 
@@ -189,10 +167,4 @@ Equivalent to perl's unshift function.
 Given a nodelist, prepends the list of nodes in $nodelist to the front of
 the current list.
 
-=head2 iterator()
-
-Will return a new nodelist iterator for the current nodelist. A
-nodelist iterator is usefull if more complex nodelist processing is
-needed.
-
 =cut
