@@ -1,24 +1,10 @@
-# $Id: Generator.pm 772 2009-01-23 21:42:09Z pajas 
-#
-# This is free software, you may use it and distribute it under the same terms as
-# Perl itself.
-#
-# Copyright 2001-2003 AxKit.com Ltd., 2002-2006 Christian Glahn, 2006-2009 Petr Pajas
-#
-#
+# $Id: Generator.pm,v 1.3 2001/11/14 10:59:58 matt Exp $
 
 package XML::LibXML::SAX::Generator;
 
 use strict;
 
 use XML::LibXML;
-use vars qw ($VERSION);
-
-$VERSION = "1.69_2"; # VERSION TEMPLATE: DO NOT CHANGE
-
-sub CLONE_SKIP {
-  return $XML::LibXML::__threads_shared ? 0 : 1;
-}
 
 warn("This class (", __PACKAGE__, ") is deprecated!");
 
@@ -142,8 +128,6 @@ XML::LibXML::SAX::Generator - Generate SAX events from a LibXML tree
 
 =head1 DESCRIPTION
 
-THIS CLASS IS DEPRACED! Use XML::LibXML::SAX::Parser instead!
-
 This helper class allows you to generate SAX events from any XML::LibXML
 node, and all it's sub-nodes. This basically gives you interop from
 XML::LibXML to other modules that may implement SAX.
